//---------------------------------------------------------------------------------------------------------------------
// Copyright (c) AUMA Riester GmbH & Co. KG (2012). All Rights Reserved
// Target System: SIMATIC PDM
//---------------------------------------------------------------------------------------------------------------------
//  (c) Sipos Aktorik GmbH, 2013 (Mathias Monse <mathias.monse@sipos.de>)
//---------------------------------------------------------------------------------------------------------------------

//----vvvvvvvv----
//#define DEVICE_IDENT   0x0C4F   // 0x0AC2: AC2_(1|2), 0x0C4F: AC2_1, 0x0CBD: AC2_2, 0x0C77: SXX
//#define DEVICE_VERSION_MAJ   3  // 99: Base version
//#define DEVICE_VERSION_MIN   0
//----^^^^^^^^----
#define	ID_CHECK_ABORT				0
#define	ID_CHECK_SUCCESS			1
#define	ID_CHECK_WRITABLE			0
#define	ID_CHECK_WRITE_PROTECTED	1
#define	ID_FC_ERROR					3000

#define FW_VERSION_DEFAULT      	99
#define IDENT_AC2_DEFAULT      		0x0AC2

#ifndef _V510_
#define LOG_MESSAGE(priority, text)  // _TRACE(text) 
#endif

#ifndef VARIABLE_STATUS_LOADED
#define VARIABLE_STATUS_LOADED        0x0010
#endif

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_version_major (internal use only)
// 
VARIABLE var_version_major	
{
    LABEL		"fw version (major)";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER (2);
    HANDLING		READ;
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  var_version_minor (internal use only)
// 
VARIABLE var_version_minor	
{
    LABEL		"fw version (minor)";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER (2);
    HANDLING		READ;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining Ident checking variables
//
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  ret_CheckDeviceIdentification
// 
VARIABLE	ret_CheckDeviceIdentification
{
	LABEL		"ret_CheckDeviceIdentification";
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER (1)
	{
		DEFAULT_VALUE	ID_CHECK_ABORT;
	}
	HANDLING	READ;
}
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining VARIABLE  ret_CheckWriteProtection
// 
VARIABLE	ret_CheckWriteProtection
{
	LABEL		"ret_CheckWriteProtection";
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER (1)
	{
		DEFAULT_VALUE	ID_CHECK_WRITABLE;
	}
	HANDLING	READ;
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD extract_version_majmin
// 
// INPAR in: version string from device variable (e.g. "03.01.02-1234")
// OUTPAR out1: extracted major version (e.g. "3")
// OUTPAR out2: extracted minor version (e.g. "1")
//
// TODO: parse and check version string: "[.+](\d\d)\.(\d\d)\.(\d\d)-(\d+)"
//       \1= major, \2= minor, \3= release, \4= build
//       and setup extracted version: "(\d\d)(\d\d)"
//       \1= major, \2= minor
#define MAX_VERSION_LENGTH    4
METHOD extract_version_majmin (DD_STRING sinput, unsigned int & major, unsigned int & minor)
{
	LABEL		"extract_version_majmin";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		int i;
		int j;
		char vers[8];
		char in[20];
		in= sinput;
		vers= "";
		j= 0;
		for (i= 0; i < strlen(in) && j < MAX_VERSION_LENGTH; i++)
		{
			if (in[i] >= '0' && in[i] <= '9')
			{
				vers[j++]= in[i];
			}
		}
		if (strlen(vers) == 4)
		{
			major= atoi(vers)/100;
			minor= atoi(vers)%100;
		}
		else
		{
			major= 0;
			minor= 0;
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD build_version_string
// 
// INPAR in1: major version as unsigned int 
// INPAR in2: minor version as unsigned int 
// OUTPAR out1: version string (e.g. "03.01")
//
METHOD build_version_string ( unsigned int major, unsigned int minor, DD_STRING &soutput )
{
	LABEL		"build_version_string";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		char sz[];
		
		if (major < 10) 
		{
			sz= "0";
		}
		sz += itoa(major);
		sz += ".";
		if (minor < 10) 
		{
			sz += "0";
		}
		sz += itoa(minor);
		soutput= sz;
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   METHOD AumaDeviceIdent
// 
//   * Method reads Profibus-Ident (PRM_3006) und Firmware-Version (PRM_1759) and calculates 
//     ret_CheckWriteProtection and ret_CheckDeviceIdentification
//
METHOD check_AUMADeviceIdent
{
	LABEL		"Identify AUMA device";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		int		iResult;
		char    sz[];
		char    msg[];
		// Algorithm:
		// * Read PROFIBUS-Ident (PRM_3006) and fw version (PRM_1759) + extract Major-Minor-Version ("[<V>]<MM>.<mm>.<rr>-<bbbbb>" --> "<MM><mm>")
		// * check compatibility:
		// -- Ident_EDD == Ident_dev... 
		//         && major_version_EDD == major_version_dev && minor_version_EDD <= minor_version_dev --> ident_ok + r/w
		//         && major_version_EDD == major_version_dev && minor_version_EDD > minor_version_dev --> ident_ok + r/o
		// -- Ident_EDD != Ident_dev || major_version_EDD != major_version_dev --> no_ident + r/o
		
		iResult = ID_FC_ERROR;

		// LOG_MESSAGE(0, "### AumaDeviceIdent: read profibus ident");
		iResult = ReadCommand(read_view_DIS_602); // contains profibus ident (PRM_3006)
		if (iResult >= ID_FC_ERROR)
		{
			// LOG_MESSAGE(0, "### AumaDeviceIdent (ERROR):  Comm error while reading device ident");
			process_abort();
		}
		// LOG_MESSAGE(0, "### AumaDeviceIdent: read firmware version");
		iResult = ReadCommand(read_view_DIS_62); // contains f/w version (PRM_1759)
		if (iResult >= ID_FC_ERROR)
		{
			// LOG_MESSAGE(0, "### AumaDeviceIdent (ERROR):  Comm error while reading device version");
			process_abort();
		}
		// workaround due to DPV1_Abort_Request
		if ((PRM_3006.VARIABLE_STATUS != VARIABLE_STATUS_LOADED) || (PRM_1759.VARIABLE_STATUS != VARIABLE_STATUS_LOADED))
		{
			// device's check not possible yet 
			// LOG_MESSAGE(0, "### AumaDeviceIdent (result): device's check not possible yet --> write protected access \n" + 
			//	"(PRM_3006.VARIABLE_STATUS= " + PRM_3006.VARIABLE_STATUS + ",\n PRM_1759.VARIABLE_STATUS= " + PRM_1759.VARIABLE_STATUS + ")");
			
			ret_CheckWriteProtection= ID_CHECK_WRITE_PROTECTED;
			ret_CheckDeviceIdentification= ID_CHECK_SUCCESS;
			return;
		}
		
		// extract fw version
		extract_version_majmin(PRM_1759, var_version_major, var_version_minor);

		// check compatibility of device
		// LOG_MESSAGE(0, "### AumaDeviceIdent (read from device): ident= " + itoa(PRM_3006) + ", version= " + PRM_1759);
		if ((PRM_3006 == DEVICE_IDENT) ||
			((DEVICE_IDENT == IDENT_AC2_DEFAULT) && ((PRM_3006 == 0x0C4F)||(PRM_3006 == 0x0CBD))))
		{
			if (FW_VERSION_DEFAULT == DEVICE_VERSION_MAJ)
			{
				// --> basic version matches always
				// LOG_MESSAGE(0, "### AumaDeviceIdent (result): EDD is of basic version --> writeable access");
				ret_CheckWriteProtection= ID_CHECK_WRITABLE;
				ret_CheckDeviceIdentification= ID_CHECK_SUCCESS;
				return;
			}
			// check if device's major version matches
			if (var_version_major == DEVICE_VERSION_MAJ)
			{
				if (var_version_minor == DEVICE_VERSION_MIN)
				{
					// --> device's version matches perfectly
					// LOG_MESSAGE(0, "### AumaDeviceIdent (result): device's version matches perfectly --> writeable access");
					ret_CheckWriteProtection= ID_CHECK_WRITABLE;
					ret_CheckDeviceIdentification= ID_CHECK_SUCCESS;
					return;
				}
				else if (var_version_minor > DEVICE_VERSION_MIN)
				{				
					// device's version is newer
					// LOG_MESSAGE(0, "### AumaDeviceIdent (result): device's version is newer --> writeable access");
					ret_CheckWriteProtection= ID_CHECK_WRITABLE;
					ret_CheckDeviceIdentification= ID_CHECK_SUCCESS;
					return;
				}
				else if (var_version_minor < DEVICE_VERSION_MIN)
				{
					// device's version is older
					// LOG_MESSAGE(0, "### AumaDeviceIdent (result): device's version is older --> write protected access");
					get_dictionary_string(TXT_IDENTCHECK_READONLY, sz);
					msg= sz;
					get_dictionary_string(TXT_IDENTCHECK_USEMATCHINGORBASIC, sz);
					msg += "\n" + sz;
					get_dictionary_string(TXT_VERSION_OF_DEV, sz);
					msg += "\n\n\t" + sz;
					build_version_string(var_version_major, var_version_minor, sz);
					msg += ":\t" + sz;
					get_dictionary_string(TXT_VERSION_OF_EDD, sz);
					msg += "\n\t" + sz;
					build_version_string(DEVICE_VERSION_MAJ, DEVICE_VERSION_MIN, sz);
					msg += ":\t" + sz;
					ACKNOWLEDGE(msg);

					ret_CheckWriteProtection= ID_CHECK_WRITE_PROTECTED;
					ret_CheckDeviceIdentification= ID_CHECK_SUCCESS;
					return;
				}
			}
		}
		// otherwise no access permitted
		// LOG_MESSAGE(0, "### AumaDeviceIdent (result): no access permitted");
		get_dictionary_string(TXT_IDENTCHECK_NOACCESS, sz);
		msg= sz;
		get_dictionary_string(TXT_IDENTCHECK_USEMATCHINGORBASIC, sz);
		msg += "\n" + sz;
		get_dictionary_string(TXT_VERSION_OF_DEV, sz);
		msg += "\n\n\t" + sz;
		build_version_string(var_version_major, var_version_minor, sz);
		msg += ":\t" + sz;
		get_dictionary_string(TXT_VERSION_OF_EDD, sz);
		msg += "\n\t" + sz;
		build_version_string(DEVICE_VERSION_MAJ, DEVICE_VERSION_MIN, sz);
		msg += ":\t" + sz;
		ACKNOWLEDGE(msg);

		ret_CheckWriteProtection= ID_CHECK_WRITE_PROTECTED;
		ret_CheckDeviceIdentification= ID_CHECK_ABORT;
		return;
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD std_CheckWriteProtection
// 
//   Description:
//   * Method is to calculate ret_CheckWriteProtection
//     (1 - r/o, 0 - r/w)
//
METHOD	std_CheckWriteProtection
{
	LABEL		"std_CheckWriteProtection";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		// LOG_MESSAGE(0, "### std_CheckWriteProtection for AC2 devices is called");
		check_AUMADeviceIdent();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD std_CheckDeviceIdentification
// 
//   Description:
//   * Method is called immediately before any online dialog is opened
//   * Method reads Profibus-Ident (PRM_3006) und Firmware-Version (PRM_1759) to calculate ret_CheckDeviceIdentification
//     (1 - device matches EDD, 0 - incompatible device)
//
METHOD std_CheckDeviceIdentification
{
	LABEL		"std_CheckDeviceIdentification";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		// LOG_MESSAGE(0, "### std_CheckDeviceIdentification for AC2 devices is called");
		check_AUMADeviceIdent();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD std_CheckDeviceIdentificationDownload
// 
//   Description:
//   * Method is called immendiately before "Load to pc" is started
//   * Method reads Profibus-Ident (PRM_3006) und Firmware-Version (PRM_1759) to calculate ret_CheckDeviceIdentification
//     (1 - device matches EDD, 0 - incompatible device, no communication allowed)
//
METHOD std_CheckDeviceIdentificationDownload
{
	LABEL		"std_CheckDeviceIdentificationDownload";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		// LOG_MESSAGE(0, "### std_CheckDeviceIdentificationDownload for AC2 devices is called");
		check_AUMADeviceIdent();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining METHOD std_CheckDeviceIdentificationUpload
// 
//   Description:
//   * Method is called immendiately before "Load to device" is started
//   * Method reads Profibus-Ident (PRM_3006) und Firmware-Version (PRM_1759) to calculate ret_CheckDeviceIdentification
//     (1 - device matches EDD, 0 - incompatible device, no communication allowed)
//
METHOD std_CheckDeviceIdentificationUpload
{
	LABEL		"std_CheckDeviceIdentificationUpload";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		// LOG_MESSAGE(0, "### std_CheckDeviceIdentificationUpload for AC2 devices is called");
		check_AUMADeviceIdent();
	}
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   End of file
//---------------------------------------------------------------------------------------------------------------------
